{% extends layout_data.main_layout|default:"genviews/layout.html" %}
{% load i18n %}

{% block content %}
<div class="heading-flex">

  <h1>
    {{ object }}
    <div class="subheading">
      <ul>
        {% for label, value in object.get_array_data.items %}
        <li>{{ label }} :
          <em>{{ value }}</em>
        </li>
        {% endfor %}
      </ul>
    </div>
  </h1>

    {% for href, faclass in detail_menu %}
    <a href = "{{ href }}">
        <i class="fas {{ faclass }}"></i>
    </a>
    {% endfor %}
</div>

<hr />

{% if obj_html %}
    {% include obj_html %}
{% endif %}

{% endblock %}
